import java.util.HashSet;
import java.util.LinkedList;
import java.util.NoSuchElementException;
import java.util.PriorityQueue;
import java.util.Stack;
import java.util.Vector;

public class Graph {

	private final int MAX_NUMBER_OF_VERTICES = 100;
	
	private LinkedList<Vertex> vertices;
	
	public Graph() {
		this.vertices = new LinkedList<Vertex>();
	}
	
	public void addVertex(Vertex v) {
		if (this.vertices.indexOf(v) == -1) {
			this.vertices.add(v);
		}
	}
	
	public void addVertices(Vertex[] verticesToAdd) {
		for ( int i = 0 ; i < verticesToAdd.length ; i++ ) {
			this.addVertex(verticesToAdd[i]);
		}
	}
	
	public boolean vertexExistsInGraph(Vertex v) {
		if (this.vertices.indexOf(v) != -1) {
			return true;
		} else {
			return false;
		}
	}
	
	public void unvisitAllVertices() {
		for ( int i = 0 ; i < this.vertices.size() ; i++ ) {
			this.vertices.get(i).setDistance(-1);
		}
	}
	
	public void breadhFirstSearch(Vertex startVertex) throws NoSuchElementException {
		if (!this.vertexExistsInGraph(startVertex)) {
			throw new NoSuchElementException("startVertex = " + startVertex + " not part of the graph");
		} else {
			HashSet<Vertex> set = new HashSet<Vertex>();
			Vector<Vertex> queue = new Vector<Vertex>();
			
			this.unvisitAllVertices();
			queue.add(startVertex);
			set.add(startVertex);
			startVertex.setDistance(0);
			
			while (!queue.isEmpty()) {
				Vertex curVertex = queue.remove(0);
//				System.out.println("breadthFirstSearch() = curVertex = " + curVertex);
				int curDistance = curVertex.getDistance();
				
				Object[] adjacentVertices = curVertex.getAdjacentVertices();
				for ( int i = 0  ; i < adjacentVertices.length ; i++ ) {
					Vertex curAdjacentVertex = (Vertex) adjacentVertices[i];
					if (!set.contains(curAdjacentVertex)) {
						curAdjacentVertex.setDistance(curDistance + 1);
						queue.add(curAdjacentVertex);
						set.add(curAdjacentVertex);
					}
				}
			}
		}
	}
	
	public void depthFirstSearch(Vertex startVertex) throws NoSuchElementException {
		if (!this.vertexExistsInGraph(startVertex)) {
			throw new NoSuchElementException("startVertex = " + startVertex + " not part of the graph");
		} else {
			Stack<Vertex> stack = new Stack<Vertex>();
			HashSet<Vertex> set = new HashSet<Vertex>();
			
			this.unvisitAllVertices();
			stack.push(startVertex);
 
			while (!stack.isEmpty()) {
				Vertex curVertex = stack.pop();
				curVertex.setDistance(0);
//				System.out.println("depthFirstSearch() = curVertex = " + curVertex);
				int curDistance = curVertex.getDistance();
				
				if (!set.contains(curVertex)) {
					set.add(curVertex);
					Object[] adjacentVertices = curVertex.getAdjacentVertices();
					for ( int i = 0  ; i < adjacentVertices.length ; i++ ) {
						Vertex curAdjacentVertex = (Vertex) adjacentVertices[i];
						stack.push(curAdjacentVertex);
					}
				}
			}
		}		
	}
	
	public void DyjkstraShortestPaTh(Vertex startVertex) throws NoSuchElementException {
		if (!this.vertexExistsInGraph(startVertex)) {
			throw new NoSuchElementException("startVertex = " + startVertex + " not part of the graph");
		} else {
			PriorityQueue<Vertex> unvisitedQueue = new PriorityQueue<Vertex>();
			
			startVertex.setDistance(0);

			for ( int i = 0 ; i < vertices.size() ; i++ ) {
				Vertex curVertex = vertices.get(i);
				if ( !curVertex.equals(startVertex) ) {
					curVertex.setDistance(Integer.MAX_VALUE);
				}
				curVertex.setPreviousVertex(null);
				unvisitedQueue.add(curVertex);
			}
			
			while (!unvisitedQueue.isEmpty())  {
				// get vertex with min distance
				Vertex curVertex = unvisitedQueue.poll();
				Edge[] edges =  curVertex.getAllEdges();
				
				for ( int i = 0 ; i < edges.length ; i++ ) {
					Vertex adjacentVertex = edges[i].getDestination();

					int edgeWeigth = edges[i].getWeight();
					int alternativeWeight = curVertex.getDistance() + edgeWeigth;
					
					if (adjacentVertex.getDistance() > alternativeWeight) {
						adjacentVertex.setDistance(alternativeWeight);
						adjacentVertex.setPreviousVertex(curVertex);
						unvisitedQueue.remove(adjacentVertex);
						unvisitedQueue.add(adjacentVertex);
					}
				}
			}
		}
	}
	
	@Override
	public String toString() {
		String s = "[Graph ";
		for ( int i = 0 ; i < this.vertices.size() ; i++ ) {
			s += "\t" + this.vertices.get(i).toString() + "\n";
		}
		s += "]";
		
		return s;
	}
}
