import java.util.LinkedList;

public class Vertex implements Comparable<Vertex> {

	private String name;
	private LinkedList<Edge> outgoingEdges;
	private int distance;
	private Vertex previousVertex;

	public Vertex(String name) {
		this.name = name;
		this.distance = -1;
		this.outgoingEdges = new LinkedList<Edge>() ;
		this.previousVertex = null;
	}
	
	public String getName() {
		return name;
	}
	
	public int getDistance() {
		return distance;
	}

	public void setDistance(int distance) {
		this.distance = distance;
	}

	public Vertex getPreviousVertex() {
		return previousVertex;
	}

	public void setPreviousVertex(Vertex previousVertex) {
		this.previousVertex = previousVertex;
	}

	public Object[] getAdjacentVertices() {
		Object[] outgoingEdges = this.outgoingEdges.toArray();
		Vertex[] adjacentVertices = new Vertex[outgoingEdges.length];
		for ( int i = 0 ; i < outgoingEdges.length ; i++ ) {
			Edge edge = (Edge) outgoingEdges[i];
			adjacentVertices[i] = edge.getDestination();
		}
		return adjacentVertices;
	}
	
	public void addEdge(Edge edge) {
			this.outgoingEdges.add(edge);
	}
	
	public void addEdges(Edge[] edges) {
		for ( int i = 0 ; i < edges.length ; i++ ) {
			this.addEdge(edges[i]);
		}
	}
	
	public Edge[] getAllEdges() {
		Edge[] edges = new Edge[this.outgoingEdges.size()];
		for ( int i = 0 ; i < edges.length ; i++ ) {
			edges[i] = this.outgoingEdges.get(i);
		}
		return edges;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof Vertex) {
			Vertex otherVertex = (Vertex) obj;
			if (otherVertex.getName().equals(this.name)) {
				return true;
			} 
		}
		
		return false;
	}

	@Override
	public String toString() {
		String s = "Vertex [ name = " + this.name +  " | distance = " + this.distance + " previous= " +  
										(this.previousVertex == null ? "null" : this.previousVertex.getName()) + " | Edges = {";
		for ( int i = 0 ; i < this.outgoingEdges.size() ; i++ ) {
			s = s + this.outgoingEdges.get(i);
		}
		
		if (this.outgoingEdges.size() == 0) {
			s = s + "  }\t]";
		} else {
			s = s + "  }\n\t]";
		}
		
		return s;
	}

	@Override
	public int compareTo(Vertex otherVertex) {
		if (this.distance < otherVertex.getDistance()) {
			return -1;
		} else if (this.distance > otherVertex.getDistance()) {
			return 1;
		} else {
			return 0;
		}
	}
}
